require "thor"
require "json"
require "leveldb"
require "securerandom"
require_relative "./request"

module RequestClient
    class Zyxt < Thor
        attr_accessor :db
        def self.exit_on_failure?
            true
        end

        desc "history", "show history of requests"
        def history
            @db = LevelDB::DB.new '/tmp/foo'
            @db.each { |key, val| puts "Key: #{key}, Val: #{val}" }
        end

        desc "get URL", "execute GET request to URL"
        option :headers, :type => :hash, :desc => "Headers"
        option :curl, :type => :boolean, :desc => "Print command as cURL"
        def get(url)
            if options[:curl]
                puts "curl -X GET #{url}"
                return nil
            end

            response = Request::Http.send_request("get", url, nil, options[:headers])
            Request::Http.log(response)
            save_request("get")
        end

        desc "post URL", "execute POST request to URL"
        option :body, :type => :hash, :desc => "Body of the request"
        option :headers, :type => :hash, :desc => "Headers"
        option :curl, :type => :boolean, :desc => "Print command as cURL"
        def post(url)
            if options[:curl]
                puts "curl -X POST #{url} -H 'Content-Type: application/json' -d #{options[:body]}"
                return nil
            end

            response = Request::Http.send_request("post", url, options[:body], options[:headers])
            Request::Http.log(response)
            save_request("post")
        end

        desc "put URL", "execute PUT request to URL"
        option :body, :type => :hash, :desc => "Body of the request"
        option :headers, :type => :hash, :desc => "Headers"
        option :curl, :type => :boolean, :desc => "Print command as cURL"
        def put(url)
            if options[:curl]
                puts "curl -X PUT #{url} -H 'Content-Type: application/json' -d #{options[:body]}"
                return nil
            end

            response = Request::Http.send_request("put", url, options[:body], options[:headers])
            Request::Http.log(response)
            save_request("put")
        end

        desc "delete URL", "execute DELETE request to URL"
        option :headers, :type => :hash, :desc => "Headers"
        option :curl, :type => :boolean, :desc => "Print command as cURL"
        def delete(url)
            if options[:curl]
                puts "curl -X DELETE #{url}"
                return nil
            end

            response = Request::Http.send_request("delete", url, nil, options[:headers])
            Request::Http.log(response)
            save_request("delete")
        end


        no_commands do
            def save_request(method)
                if @db.nil?
                    @db = LevelDB::DB.new '/tmp/foo'
                end

                tempHash = {"hash" => "Command executed #{method}", "executed_at" => Time.now}

                @db.put(SecureRandom.hex, tempHash)
            end
        end
    end
end

RequestClient::Zyxt.start(ARGV)
